;+
; NAME:
;	CHECK_DATA_PARTEMIS
;
; PURPOSE:
;	Check if data structure read from MB-FITS file is non-empty and correct. It returns 1 for correct structure, -1 for non correct structure	
;
; CALLING SEQUENCE:
;	Check_data = CHECK_DATA_PARTEMIS(Datastr)
;
; INPUTS:
;	Datastr:	Data structure.
;	
; COMMON BLOCKS:
;	OBS1_CONFIGB:	Contains working directory, project and calibration table name.
;		
; EXAMPLE:
;		Check_data = CHECK_DATA_PARTEMIS(Datastr)
;
; MODIFICATION HISTORY: 	
; 	2014-2016 : P. Gallais  - created
;   201710    : F. Schuller - cleaning / rewriting
;-

function check_data_partemis, datastr

COMMON obs1_configb

  if datatype(datastr) NE 'STC' then begin
    print, 'ERREUR : DATASTR N EST PAS UNE STRUCTURE'
    return, -1
  endif

  ; Init message to empty string
  message = ''
  
  ; Test various possible errors, update message if any
  dimens = size(datastr.cube)
  if dimens[0] NE 3 then $
    message = message + 'ERREUR : DATASTR.CUBE NE COMPORTE PAS 3 MAIS ' + strcompress(string(dimens[0]))+ ' DIMENSIONS'

  if datastr.elevatio gt 90. OR  datastr.elevatio lt 0. then $
    message = message + 'ERREUR : DATASTR.ELEVATIO N EST PAS COMPRIS ENTRE 0 ET 90 DEGRES ET VAUT ' + strcompress(string(datastr.elevatio))

  if datastr.sitelat gt 90. OR  datastr.sitelat lt -90. then $
    message = message + 'ERREUR : DATASTR.SITELAT N EST PAS COMPRIS ENTRE -90 ET 90 DEGRES ET VAUT ' + strcompress(string(datastr.sitelat))

  if datastr.sitelong gt 360. OR  datastr.sitelong lt -360. then $
    message = message + 'ERREUR : DATASTR.SITELONG N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET VAUT ' + strcompress(string(datastr.sitelong))

  if datastr.blongobj gt 360. OR  datastr.blongobj lt -360. then $
    message = message + 'ERREUR : DATASTR.BLONGOBJ N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET VAUT ' + strcompress(string(datastr.blongobj))

  if datastr.blatobj gt 360. OR  datastr.blatobj lt -360. then $
    message = message + 'ERREUR : DATASTR.BLATOBJ N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET VAUT' + ' ' + strcompress(string(datastr.blatobj))

  if datastr.longobj gt 360. OR  datastr.longobj lt -360. then $
    message = message + 'ERREUR : DATASTR.LONGOBJ N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET VAUT ' + strcompress(string(datastr.blongobj))

  if datastr.latobj gt 360. OR  datastr.latobj lt -360. then $
    message = message + 'ERREUR : DATASTR.LATOBJ N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET VAUT ' + strcompress(string(datastr.latobj))

  if datastr.wobused NE 1 AND  datastr.wobused NE 0 then $
    message = message + 'ERREUR : DATASTR.WOBUSED EST DIFFERENT DE 0 OU 1 ET VAUT ' + strcompress(string(datastr.wobused))

  if datastr.mjd gt 100000. OR  datastr.mjd lt 50000. then $
    message = message + 'ERREUR : DATASTR.MJD EST ABERRANT ET VAUT ' + strcompress(string(datastr.mjd))

  ind = where(datastr.lst ne -999., cnt)
  if cnt gt 0 then begin
    lst = (datastr.lst)(ind)
    if max(lst) gt 25.*3600. OR  min(lst) lt 0. then $
      message = message + 'ERREUR : DATASTR.LST EST ABERRANT ET EST COMPRIS ENTRE ' + $
                           strcompress(string(min(lst))) + ' -- ' + strcompress(string(max(lst)))
  endif

  if max(datastr.azim) gt 360. OR  min(datastr.azim) lt -360. then $
    message = message + 'ERREUR : DATASTR.AZIM EST ABERRANT ET EST COMPRIS ENTRE ' + $
                         strcompress(string(min(datastr.azim))) + ' ' + strcompress(string(max(datastr.azim)))

  if max(datastr.elev) gt 90. OR  min(datastr.elev) lt 0. then $
    message = message + 'ERREUR : DATASTR.ELEV EST ABERRANT ET EST COMPRIS ENTRE ' + $
                         strcompress(string(min(datastr.elev))) + ' ' + strcompress(string(max(datastr.elev)))

  if datastr.crval1 gt 360. OR  datastr.crval1 lt -360. then $
    message = message + 'ERREUR : DATASTR.CRVAL1 N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET VAUT ' + strcompress(string(datastr.crval1))

  if datastr.crval2 gt 360. OR  datastr.crval2 lt -360. then $
    message = message + 'ERREUR : DATASTR.CRVAL2 N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET VAUT ' + strcompress(string(datastr.crval2))

  ind = where(datastr.datapar.elevatio ne -999., cnt)
  if cnt gt 0 then begin
    elevatio = (datastr.datapar.elevatio)(ind)
    if max(elevatio) gt 90. OR  min(elevatio) lt 0. then $
      message = message + 'ERREUR : DATASTR.DATAPAR.ELEVATIO N EST PAS COMPRIS ENTRE 0 ET 90 DEGRES ET S INSCRIT DANS L INTERVALLE ' + $
                           strcompress(string(min(elevatio))) + ' ' + strcompress(string(max(elevatio)))
  endif

  if max(datastr.datapar.mjd) gt 100000. OR  min(datastr.datapar.mjd) lt 50000. then $
    message = message + 'ERREUR : DATASTR.DATAPAR.MJD EST ABERRANT ET S INSCRIT DANS L INTERVALLE ' + $
                         strcompress(string(min(datastr.datapar.mjd))) + ' ' + strcompress(string(max(datastr.datapar.mjd)))

  ind = where(datastr.datapar.azimuth ne -999., cnt)
  if cnt gt 0 then begin
    azimuth = (datastr.datapar.azimuth)(ind)
    if max(azimuth) gt 360. OR  min(azimuth) lt -360. then $
      message = message + 'ERREUR : DATASTR.DATAPAR.AZIMUTH N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET S INSCRIT DANS L INTERVALLE ' + $
                           strcompress(string(min(azimuth))) + ' ' + strcompress(string(max(azimuth)))
  endif	

  ind = where(datastr.datapar.parangle ne -999., cnt)
  if cnt gt 0 then begin
    parangle = (datastr.datapar.parangle)(ind)
    if max(parangle) gt 360. OR  min(parangle) lt -360. then $
      message = message + 'ERREUR : DATASTR.DATAPAR.PARANGLE N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET S INSCRIT DANS L INTERVALLE ' + $
                          strcompress(string(min(datastr.datapar.parangle))) + ' ' + strcompress(string(max(datastr.datapar.parangle)))
  endif

  ind = where(datastr.datapar.cbaslong ne -999., cnt)
  if cnt gt 0 then begin
    cbaslong = (datastr.datapar.cbaslong)(ind)
    if max(cbaslong) gt 360. OR  min(cbaslong) lt -360. then $
      message = message + 'ERREUR : DATASTR.DATAPAR.CBASLONG N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET S INSCRIT DANS L INTERVALLE ' + $
                           strcompress(string(min(cbaslong))) + ' ' + strcompress(string(max(cbaslong)))
  endif	

  ind = where(datastr.datapar.cbaslat ne -999., cnt)
  if cnt gt 0 then begin
    cbaslat = (datastr.datapar.cbaslat)(ind)
    if max(cbaslat) gt 360. OR  min(cbaslat) lt -360. then $
      message = message + 'ERREUR : DATASTR.DATAPAR.CBASLAT N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET S INSCRIT DANS L INTERVALLE ' + $
                           strcompress(string(min(cbaslat))) + ' ' + strcompress(string(max(cbaslat)))
  endif

  ind = where(datastr.datapar.baslong ne -999., cnt)
  if cnt gt 0 then begin
    baslong = (datastr.datapar.baslong)(ind)
    if max(baslong) gt 360. OR  min(baslong) lt -360. then $
      message = message + 'ERREUR : DATASTR.DATAPAR.BASLONG N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET S INSCRIT DANS L INTERVALLE ' + $
                           strcompress(string(min(baslong))) + ' ' + strcompress(string(max(baslong)))
  endif	

  ind = where(datastr.datapar.baslat ne -999., cnt)
  if cnt gt 0 then begin
    baslat = (datastr.datapar.baslat)(ind)
    if max(baslat) gt 360. OR  min(baslat) lt -360. then $
      message = message + 'ERREUR : DATASTR.DATAPAR.BASLAT N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET S INSCRIT DANS L INTERVALLE ' + $
                           strcompress(string(min(baslat))) + ' ' + strcompress(string(max(baslat)))
  endif

  ind = where(datastr.datapar.ra ne -999., cnt)
  if cnt gt 0 then begin
    ra = (datastr.datapar.ra)(ind)
    if max(ra) gt 360. OR  min(ra) lt -360. then $
      message = message + 'ERREUR : DATASTR.DATAPAR.RA N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET S INSCRIT DANS L INTERVALLE ' + $
                           strcompress(string(min(ra))) + ' ' + strcompress(string(max(ra)))
  endif	

  ind = where(datastr.datapar.dec ne -999., cnt)
  if cnt gt 0 then begin
    dec = (datastr.datapar.dec)(ind)
    if max(dec) gt 360. OR  min(dec) lt -360. then $
      message = message + 'ERREUR : DATASTR.DATAPAR.DEC N EST PAS COMPRIS ENTRE -360 ET 360 DEGRES ET S INSCRIT DANS L INTERVALLE ' + $
                           strcompress(string(min(dec))) + ' ' + strcompress(string(max(dec)))
  endif

  if message eq '' $
    then return,1 $
    else begin
      print,message
      openW, lun, work_dir + 'apexdata/' + strcompress('Error_report_on_' + string(datastr.scannum)) + '_data.txt' , /APPEND, /GET_LUN
      printf, lun, "Subscan: "+strtrim(datastr.subscan,2), message
      close, lun
      free_lun, lun
      return, -1
  endelse
end
