;+
; NAME:
;	INIT_OBS
;
; PURPOSE:
;	Look for input raw data file(s), and fill a structure containing directory path,
;	calibration file,	and subscan list for a given observation ID.
;
; CALLING SEQUENCE:
;	INIT_OBS, Scan_number=Scan_number, Type=Type, Init_obs_str, band=band
;
; INPUTS:
;	Scan_number:	Observation ID.
;	Type:		MAP, CHMAP or DECORREL.
;
; OUTPUTS:
;	Init_obs_str:	Output structure.
;
; COMMON BLOCKS:
;	OBS1_CONFIGB:	Contains working directory, project and calibration table name
;		
; EXAMPLE:
;		INIT_OBS, Scan_number=47822, Type='MAP', Init_obs_str
;
; MODIFICATION HISTORY:
;  - 2014     - P. Gallais  - created
;  - 2016     - P. Gallais  - 'band' argument added
;  - 2016     - Ph. Andre   - rot_angle, xref_pix, yref_pix added to output structure
;  - 20170908 - F. Schuller - major cleaning, and adapted to single FITS file
;  - 20171029 - Ph. Andr - fixed a bug introduced by previous major cleaning (when band=450, a call to obs1_artemis_config is needed to point to the proper calibration_table)
;-

pro init_obs, scan_number=scan_number, type=type, init_obs_str, band=band
  if n_params() eq 0 then begin
	  print, 'Calling sequence :'
    print, '	INIT_OBS, Scan_number=Scan_number, Type=Type, Init_obs_str, band=band
    print, '                  default [band=350]'
    return
  endif

;;  COMMON obs1_configb, work_dir, project_name, calibration_table          	    ; chargement des common variables depuis obs1_config
  if not keyword_set(band) then spectral_band = 350 else spectral_band = band
  @obs1_artemis_config

; Definition of input directories etc.
  obsname = 'APEX-' + strtrim(string(scan_number),1)
  apexdata = work_dir+'apexdata/'					    ; contient les donnees
  Calib_partemis = work_dir+'Calib_partemis/'                         ; contient les fichiers de calibration
  dirdat = apexdata + 'rawdata/'                                      ; contient les donnees brutes en format MBFITS
  dirdef = apexdata + 'basic_xdr/' + strtrim(long(spectral_band), 2) + '/'   ; contient les donnees brutes(V) ou etalonnees(pW) en format XDR (IDL structure)

; First, retrieve the observation date from the file name
; Look for data files in basic_xdr
  u = file_search(dirdef+'*'+strtrim(string(scan_number),1)+'*', count=nfiles)
  if u[0] EQ '' then begin
    ; if no file found, look in rawdata/
    u = file_search(dirdat+'*'+strtrim(string(scan_number),1)+'*') ; rawdata for this scan number
    if u[0] EQ '' then date='' else begin
      f_first = u[0]
      pos1 = strpos(f_first,'APEX')
      date = strmid(f_first, pos1, 30)
      date = strsplit(date, /EXTRACT, '-')
      date = date[2:4]
    endelse
  endif else begin
    ; retrieve info from existing .xdr file
    restore, u[0]
    date=strsplit(datastr.date_obs, /EXTRACT, 'T')
    date=date[0]
    date=strsplit(date, /EXTRACT, '-')
  endelse

; Now, build the list of available subscans
  subscan_liste = []
; Look for existing .xdr files
  basic_name = dirdef + '*' +'_' + strtrim(string(scan_number),1)
  file_list  = file_search(basic_name + '_*', count=nfiles)
  if nfiles gt 0 then begin
    ; at least one .xdr file present, we build the list of subscans from the files names in basic_xdr/
    print,'INFO: Found the following files in '+dirdef
    for i = 0, nfiles-1 do begin
      print, " ... ",file_list[i]
      tmp  = strsplit(file_list[i],'_',/extract,count=n_substr)
      snum = tmp[n_substr-1]                  ; end of file name, format: ???.xdr where ??? is the subscan number
      snum = strmid(snum,0,strlen(snum)-4)    ; remove '.xdr'
      subscan_liste = [subscan_liste,snum]    ; append to the list of subscans
    endfor
  ;
  endif else begin   ; les fichiers basic_xdr n'existent pas, on contruit la subscan_liste a partir des rawdata
    basic_name = dirdat + 'APEX-'+ strtrim(string(scan_number),1)+'-*'
    file_list  = file_search(basic_name, count=nfiles)
    if nfiles eq 0 then begin
      print,'No .xdr file and no raw data found for scan '+strtrim(string(scan_number),1)
      print,' ... exiting.'
      init_obs_str = {status:0}   ; result = 0
      return
    endif
    if file_test(file_list[0], /dir) then begin
      ; the raw data is stored in the MB-FITS direcory structure
      f_list = file_search(file_list[0]+'/*', count=nfiles)
      for i=0,nfiles-1 do begin
        if file_test(f_list[i], /dir) then begin
          tmp  = strsplit(f_list[i],'/',/extract,count=n_substr)
          snum = tmp[n_substr-1]                  ; last sub-dir in file name = subscan number
          subscan_liste = [subscan_liste,snum]    ; append to the list of subscans
        endif
      endfor
    endif else begin
      ; The raw data is in a single .fits file: we need to read the SCAN table to find the number of subscans
      scan = mrdfits(file_list[0], 1, header)
      nsubs = fxpar(header, 'NSUBS')
      subscan_liste = strtrim(string(indgen(nsubs)+1),1)   ; this assumes that all subscans are present and readable...
    endelse
  endelse
  ; Sort the list by increasing number
  if n_elements(subscan_liste) gt 0 then subscan_liste = subscan_liste(sort(long(subscan_liste)))
  subscan_name = project_name +'/' + strtrim(string(scan_number), 2) + '_' + subscan_liste

  ; Read calibration-related information
  ; to do: clean up this code...
  calibration_table2=Calib_partemis+calibration_table   ; le fichier calibration_table est place dans Calib_partemis
  openr, lun, calibration_table2 ,/GET_LUN              ; ouverture du fichier
  nlines=File_lines(calibration_table2)                 ; lecture du nombre de lignes
  table=strarr(nlines)                                  ; generation d'une table de caracteres de longueur nlines
  readf, lun, table                                     ; on stocke la lecture du fichier calibration_table dans table
  compteur=0
  number=['0','1','2','3','4','5','6','7','8','9']

  for i=0, nlines-1  do begin 
    v=strpos(number,strmid(strcompress(table(i)),0,1)) 
    if strmid(strcompress(table(i)),0,1) eq '' then begin
      compteur = compteur+1
    endif else $
      if total(v) eq -10 then begin
	      compteur = compteur + 1
      endif
  endfor

  table2=strarr(nlines-compteur,10)                 ; deuxieme table qui va ne contenir que les lignes utiles, et 8 colonnes pour
                                                    ; scan_number, Conv, goodpix_ima, flatfield, calibration_camera et rcp_file 

  for i=0, nlines-compteur-1  do begin
    table(compteur+i)=strjoin(strsplit(table(compteur+i),/EXTRACT),'-')   ; pour une ligne utile, les blancs sont elimines et remplaces par un '-'
    table2(i,*)=strsplit(table(compteur+i),/EXTRACT,'-')                  ; extraction des sous chaines de caracteres (5 par ligne)
  endfor

  scan_table=long(table2(*,0))              ; conversion string en entier du scan_number
                                           ; contient la liste des scan_number dont nous connaissons quelques informations
  min_index=min(where(scan_number le scan_table))
  max_index=max(where(scan_number ge scan_table))
  if (where(scan_number eq scan_table) ne -1) then begin
    index_c=where(scan_number eq scan_table)     ; si le scan_number se trouve dans la scan_table 
  endif else begin
    ; si le scan_number ne se trouve pas dans la scan_table (cas le plus frequent)
    ; on prend la ligne correspondant au numero de scan le plus proche inferieurement.
    index_c=min([min_index,max_index])
  endelse

  print, "INFO: Restoring the appropriate calibration tables..."
  print, " ... Restoring ", Calib_partemis+table2(index_c,2)
  restore, Calib_partemis+table2(index_c,2)                       ; restoration de goodpix_ima, flat [(64,36)] et affectation de Conv
;
  print, " ... Restoring ", Calib_partemis+table2(index_c,3)
  restore, Calib_partemis+table2(index_c,3)
;
  print, " ... Restoring ", Calib_partemis+table2(index_c,5)
  restore, Calib_partemis+table2(index_c,5)                       ; restoration du fichier rcp:  2 x (64,36) arrays


; Introduction des faux pixels inter-matrices     [(64,36) --> (73,39) arrays]
  if long(spectral_band) eq 450 then begin
    goodpix_ima = img2focal450(goodpix_ima)	    ;  (73,39)	
    flat_field = img2focal450(flat_field)		    ;  (73,39)
    dx_rcp = img2focal450(dx_rcp, fill='NaN')	    ;  (73,39)		
    dy_rcp = img2focal450(dy_rcp, fill='NaN')	    ;  (73,39)		
  endif else begin
    goodpix_ima = img2focal350(goodpix_ima)	    ;  (73,39)	
    flat_field = img2focal350(flat_field)		    ;  (73,39)
    dx_rcp = img2focal350(dx_rcp, fill='NaN')	    ;  (73,39)		
    dy_rcp = img2focal350(dy_rcp, fill='NaN')	    ;  (73,39)		
  endelse

  calibration_camera=Calib_partemis+table2(index_c,4);
  pfov=float(table2(index_c,6))               
  pfov=pfov[0]                                ; extraction de la taille du pixel                             

  rot_angle = float(table2(index_c,7))	    ;  Rotation angle of the array in the focal plane (deg)
  rot_angle = rot_angle[0]

  x_ref_pix = float(table2(index_c,8))	    ;  x coordinate of reference pixel (for pointing/astrometry)
  x_ref_pix = x_ref_pix[0]

  y_ref_pix = float(table2(index_c,9))	    ;  Y coordinate of reference pixel (for pointing/astrometry)
  y_ref_pix = y_ref_pix[0]

; remplissage de la structure
  Conv=float(table2(index_c,1))*0.0001
  Conv=Conv[0]
  if type ne ('chmap') then begin             ; type MAP ou DECORREL 
    ;PG2016 : ajout du champ 'band'     PhA2016: ajout des champs rot_angle, xref_pix, yref_pix
    init_obs_str = {scan_number:scan_number, type:type, band:spectral_band, $
                    work_dir:work_dir, project_name:project_name, apexdata:apexdata, dirdat:dirdat, $
                    subscan_liste:subscan_liste, subscan_name:subscan_name, $
                    goodpix_ima:goodpix_ima, flat_field:flat_field, Conv:Conv, $
                    dx_rcp:dx_rcp, dy_rcp:dy_rcp, calibration_camera:calibration_camera, obsname:obsname, $
                    pfov:pfov, rot_angle:rot_angle, xref_pix: x_ref_pix, yref_pix: y_ref_pix, date:date, $
                    status:1}
  endif else begin                            ; type CHMAP
    init_obs_str = {scan_number:scan_number, type:type, band:spectral_band, $
                    work_dir:work_dir, project_name:project_name, apexdata:apexdata, dirdat:dirdat, $
                    subscan_liste:subscan_liste, subscan_name:subscan_name, $
                    goodpix_ima:goodpix_ima, flat_field:flat_field, Conv:Conv, $
                    calibration_camera:calibration_camera, obsname:obsname, pfov:pfov, date:date, $
                    status:1}
  endelse
  free_lun, lun
end
